const { Builder, By, until } = require("selenium-webdriver");
const assert = require("assert");
//const { after } = require("node:test");

//const { createUser } = require("../backend/userapi");

describe("User Managment Test", function () {
  let driver;

  beforeEach(async function () {
    driver = await new Builder().forBrowser("chrome").build();
    //navigate to the application
    await driver.get("http://localhost:3000");
    await driver.manage().window().maximize();

    await driver.findElement(By.className("loginButton")).click();
    await driver.wait(until.elementLocated(By.className("loginDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.className("loginDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.className("inputEmail")).sendKeys("john@some-host.de");
    await driver.findElement(By.className("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.className("okButton")).click();

    await driver.wait(until.elementLocated(By.className("logoutButton")), 5000);
    const isLogoutButtonOpen = await driver.findElement(By.className("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpen, true, "Der Logout Button wird nicht angezeigt.");
    
    //click UserManagment
    driver.findElement(By.linkText("UserManagment")).click();
    await driver.wait(until.elementLocated(By.className("listUsers")), 5000);
    const listUsers = await driver.findElement(By.className("listUsers")).isDisplayed();
    assert.strictEqual(listUsers, true, "Der Dialog wurde nicht geöffnet.");
  },10000);

  afterEach(async function () {
    await driver.quit();
  });

  /*
  after(async function (){
    await createUser("Flo", "flo@some-host.de","12abcAB!", false);
  });
  */

  it("listing all users and admins", async function (){
    const userJohn = await driver.findElement(By.className("user-John")).isDisplayed();
    assert.strictEqual(userJohn, true, "Der User John wurde nicht gefunden.");
    const userFlo = await driver.findElement(By.className("user-Flo")).isDisplayed();
    assert.strictEqual(userFlo, true, "Der User Flo wurde nicht gefunden.");
  },20000);

  it("create user", async function (){
    driver.findElement(By.className("addUserButton ms-3 mb-3")).click();
    await driver.wait(until.elementLocated(By.className("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.className("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.className("inputUsername")).sendKeys("Max");
    await driver.findElement(By.className("inputEmail")).sendKeys("max@some-host.de");
    await driver.findElement(By.className("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.className("addButton")).click();

    await driver.wait(until.elementLocated(By.className("user-Max")), 5000);
    const userMax = await driver.findElement(By.className("user-Max")).isDisplayed();
    assert.strictEqual(userMax, true, "Der User Max wurde nicht gefunden.");
  },10000);

  it("create admin", async function (){
    driver.findElement(By.className("addUserButton ms-3 mb-3")).click();
    await driver.wait(until.elementLocated(By.className("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.className("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.className("inputUsername")).sendKeys("MaxAdmin");
    await driver.findElement(By.className("inputEmail")).sendKeys("maxAdmin@some-host.de");
    await driver.findElement(By.className("inputPassword")).sendKeys("12abcAB!");
    await driver.findElement(By.id("formAdminCheckbox")).click();

    await driver.findElement(By.className("addButton")).click();

    await driver.wait(until.elementLocated(By.className("user-MaxAdmin")), 5000);
    const userMaxAdmin = await driver.findElement(By.className("user-MaxAdmin")).isDisplayed();
    assert.strictEqual(userMaxAdmin, true, "Der Admin Max wurde nicht gefunden.");
  },10000);

  it("edit user", async function (){
    driver.findElement(By.className("editButton-Flo")).click();
    await driver.wait(until.elementLocated(By.className("editUserDialog")), 5000);
    const isDialogOpen = await driver.findElement(By.className("editUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    const usernameInput = await driver.findElement(By.className("inputUsername"));
    await usernameInput.clear(); 
    await usernameInput.sendKeys("Florian");
    const emailInput = await driver.findElement(By.className("inputEmail"));
    await emailInput.clear(); 
    await emailInput.sendKeys("florian@some-host.de")

    await driver.findElement(By.className("editNow")).click();

    await driver.wait(until.elementLocated(By.className("user-Florian")), 5000);
    const userFlorianAdmin = await driver.findElement(By.className("user-Florian")).isDisplayed();
    assert.strictEqual(userFlorianAdmin, true, "Der User Florian wurde nicht gefunden.");

    driver.sleep(2000);

    driver.findElement(By.className("editButton-Florian")).click();
    await driver.wait(until.elementLocated(By.className("editUserDialog")), 5000);
    const isDialogOpen2 = await driver.findElement(By.className("editUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen2, true, "Der Dialog wurde nicht geöffnet.");

    const usernameInputFlorian = await driver.findElement(By.className("inputUsername"));
    await usernameInputFlorian.clear(); 
    await usernameInputFlorian.sendKeys("Flo");
    const emailInputFlorian = await driver.findElement(By.className("inputEmail"));
    await emailInputFlorian.clear(); 
    await emailInputFlorian.sendKeys("flo@some-host.de")
  },10000);

  it("delete user", async function (){
    await driver.findElement(By.className("deleteButton-Max")).click();
    await driver.sleep(500);

    // Überprüfen das "Max" nicht mehr in der Benutzerliste vorhanden ist
    const isMaxDeleted = await driver.findElement(By.className("user-Max")).isDisplayed().then(
      () => false,
      (error) => error.toString().includes("no such element"),
    );
    assert.strictEqual(isMaxDeleted, true, "Der Benutzer 'Max' wurde nicht gelöscht.");
  },10000);

  it("login with new account", async function (){
    driver.findElement(By.className("addUserButton ms-3 mb-3")).click();
    await driver.wait(until.elementLocated(By.className("addUserDialog")), 5000);

    const isAddDialogOpen = await driver.findElement(By.className("addUserDialog")).isDisplayed();
    assert.strictEqual(isAddDialogOpen, true, "Der Dialog wurde nicht geöffnet.");
    //Informationen in Dialog eintragen
    await driver.findElement(By.className("inputUsername")).sendKeys("Max");
    await driver.findElement(By.className("inputEmail")).sendKeys("max@some-host.de");
    await driver.findElement(By.className("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.className("addButton")).click();

    //Check ob User erstellt
    await driver.wait(until.elementLocated(By.className("user-Max")), 5000);
    const userMax = await driver.findElement(By.className("user-Max")).isDisplayed();
    assert.strictEqual(userMax, true, "Der User Max wurde nicht gefunden.");

    //Ausloggen
    await driver.findElement(By.className("logoutButton")).click();
    console.log("Logout erfolgreich");
    await driver.sleep(500);

    //Login mit neuen User
    await driver.findElement(By.className("loginButton")).click();
    await driver.wait(until.elementLocated(By.className("loginDialog")), 5000);
    const isLoginDialogOpen = await driver.findElement(By.className("loginDialog")).isDisplayed();
    assert.strictEqual(isLoginDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    const emailInput = await driver.findElement(By.className("inputEmail"));
    const passwordInput = await driver.findElement(By.className("inputPassword"));
    await emailInput.sendKeys("max@some-host.de");
    await passwordInput.sendKeys("12abcAB!");
    await driver.findElement(By.className("okButton")).click();


    //Check ob eingeloggt
    await driver.wait(until.elementLocated(By.className("logoutButton")), 5000);
    const isLogoutButtonOpenMax = await driver.findElement(By.className("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpenMax, true, "Der Logout Button Max wird nicht angezeigt.");
  },10000);

  it("create User without password ERROR!", async function (){
    driver.findElement(By.className("addUserButton ms-3 mb-3")).click();
    await driver.wait(until.elementLocated(By.className("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.className("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.className("inputUsername")).sendKeys("Tim");
    await driver.findElement(By.className("inputEmail")).sendKeys("tim@some-host.de");
    await driver.findElement(By.className("inputPassword")).sendKeys("");

    await driver.findElement(By.className("addButton")).click();

    await driver.wait(until.elementLocated(By.className("warningMessage")), 5000);
    const warningMessage = await driver.findElement(By.className("warningMessage")).isDisplayed();
    assert.strictEqual(warningMessage, true, "Die Fehlermeldung wurde nicht gefunden.");
  },10000);
});
